@@ -362,6 +362,7 @@ newPop = function(rawPop,mother=NULL,father=NULL,origM=NULL,
   if(is.null(simParam)){
     simParam = get("SP",envir=.GlobalEnv)
   }
+  stopifnot(sapply(simParam$genMap,length)==rawPop@nLoci)
   lastId = simParam$lastId
   id = (1:rawPop@nInd) + lastId
   lastId = max(id)
@@ -455,7 +456,7 @@ newPop = function(rawPop,mother=NULL,father=NULL,origM=NULL,
 #' pop = newPop(founderPop, simParam=SP)
 #' 
 #' #Rescale to set mean to 1
-#' SP$rescaleTraits(pop,mean=1)
+#' SP$rescaleTraits(mean=1)
 #' pop = resetPop(pop, simParam=SP)
 #' 
 #' @export
