@@ -11,8 +11,10 @@
 extern SEXP _AlphaSimR_calcChrFreq(SEXP);
 extern SEXP _AlphaSimR_calcCoef(SEXP, SEXP);
 extern SEXP _AlphaSimR_calcGenParam(SEXP, SEXP, SEXP);
+extern SEXP _AlphaSimR_callFastRRBLUP(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP _AlphaSimR_callRRBLUP(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP _AlphaSimR_callRRBLUP_D(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
+extern SEXP _AlphaSimR_callRRBLUP_D2(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP _AlphaSimR_callRRBLUP_GCA(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP _AlphaSimR_callRRBLUP_GCA2(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP _AlphaSimR_callRRBLUP_MV(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
@@ -33,6 +35,7 @@ extern SEXP _AlphaSimR_getHaplo(SEXP, SEXP, SEXP);
 extern SEXP _AlphaSimR_getHybridGv(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP _AlphaSimR_getIbdHaplo(SEXP, SEXP, SEXP);
 extern SEXP _AlphaSimR_getIbdRecHist(SEXP, SEXP, SEXP);
+extern SEXP _AlphaSimR_getNumThreads();
 extern SEXP _AlphaSimR_getOneHaplo(SEXP, SEXP, SEXP, SEXP);
 extern SEXP _AlphaSimR_getOneHaploT(SEXP, SEXP, SEXP, SEXP);
 extern SEXP _AlphaSimR_MaCS(SEXP, SEXP, SEXP, SEXP, SEXP);
@@ -43,8 +46,6 @@ extern SEXP _AlphaSimR_packHaplo(SEXP, SEXP, SEXP);
 extern SEXP _AlphaSimR_popVar(SEXP);
 extern SEXP _AlphaSimR_sampAllComb(SEXP, SEXP, SEXP);
 extern SEXP _AlphaSimR_sampHalfDialComb(SEXP, SEXP);
-extern SEXP _AlphaSimR_tuneTraitA(SEXP, SEXP, SEXP, SEXP, SEXP);
-extern SEXP _AlphaSimR_tuneTraitAD(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP _AlphaSimR_writeASGenotypes(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP _AlphaSimR_writeASHaplotypes(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP _AlphaSimR_writeGeno(SEXP, SEXP, SEXP, SEXP);
@@ -55,8 +56,10 @@ static const R_CallMethodDef CallEntries[] = {
     {"_AlphaSimR_calcChrFreq",       (DL_FUNC) &_AlphaSimR_calcChrFreq,        1},
     {"_AlphaSimR_calcCoef",          (DL_FUNC) &_AlphaSimR_calcCoef,           2},
     {"_AlphaSimR_calcGenParam",      (DL_FUNC) &_AlphaSimR_calcGenParam,       3},
+    {"_AlphaSimR_callFastRRBLUP",    (DL_FUNC) &_AlphaSimR_callFastRRBLUP,     7},
     {"_AlphaSimR_callRRBLUP",        (DL_FUNC) &_AlphaSimR_callRRBLUP,         6},
     {"_AlphaSimR_callRRBLUP_D",      (DL_FUNC) &_AlphaSimR_callRRBLUP_D,       7},
+    {"_AlphaSimR_callRRBLUP_D2",     (DL_FUNC) &_AlphaSimR_callRRBLUP_D2,     12},
     {"_AlphaSimR_callRRBLUP_GCA",    (DL_FUNC) &_AlphaSimR_callRRBLUP_GCA,     7},
     {"_AlphaSimR_callRRBLUP_GCA2",   (DL_FUNC) &_AlphaSimR_callRRBLUP_GCA2,   12},
     {"_AlphaSimR_callRRBLUP_MV",     (DL_FUNC) &_AlphaSimR_callRRBLUP_MV,      7},
@@ -77,6 +80,7 @@ static const R_CallMethodDef CallEntries[] = {
     {"_AlphaSimR_getHybridGv",       (DL_FUNC) &_AlphaSimR_getHybridGv,        6},
     {"_AlphaSimR_getIbdHaplo",       (DL_FUNC) &_AlphaSimR_getIbdHaplo,        3},
     {"_AlphaSimR_getIbdRecHist",     (DL_FUNC) &_AlphaSimR_getIbdRecHist,      3},
+    {"_AlphaSimR_getNumThreads",     (DL_FUNC) &_AlphaSimR_getNumThreads,      0},
     {"_AlphaSimR_getOneHaplo",       (DL_FUNC) &_AlphaSimR_getOneHaplo,        4},
     {"_AlphaSimR_getOneHaploT",      (DL_FUNC) &_AlphaSimR_getOneHaploT,       4},
     {"_AlphaSimR_MaCS",              (DL_FUNC) &_AlphaSimR_MaCS,               5},
@@ -87,8 +91,6 @@ static const R_CallMethodDef CallEntries[] = {
     {"_AlphaSimR_popVar",            (DL_FUNC) &_AlphaSimR_popVar,             1},
     {"_AlphaSimR_sampAllComb",       (DL_FUNC) &_AlphaSimR_sampAllComb,        3},
     {"_AlphaSimR_sampHalfDialComb",  (DL_FUNC) &_AlphaSimR_sampHalfDialComb,   2},
-    {"_AlphaSimR_tuneTraitA",        (DL_FUNC) &_AlphaSimR_tuneTraitA,         5},
-    {"_AlphaSimR_tuneTraitAD",       (DL_FUNC) &_AlphaSimR_tuneTraitAD,        7},
     {"_AlphaSimR_writeASGenotypes",  (DL_FUNC) &_AlphaSimR_writeASGenotypes,   7},
     {"_AlphaSimR_writeASHaplotypes", (DL_FUNC) &_AlphaSimR_writeASHaplotypes,  7},
     {"_AlphaSimR_writeGeno",         (DL_FUNC) &_AlphaSimR_writeGeno,          4},
